# Purpose:  add GUI components to base classes
#
# Notes:	add some GUI functionality to otherwise independent classes.  doing this here
#					lets us keep good model/view separation, but avoid having to go through a third
# 				party to perform basic functions.
#

require 'fox12'

require 'SessionManager'

if RUBY_PLATFORM =~ /win32/
	begin
		require 'Win32API'
	rescue
	end
end

include Fox

module Mondrian
	class FXFileDialog < FXFileDialog
		attr_reader :filenames
		
		@@get_file_name 		= nil
		@@get_save_name			= nil
		@@get_active_window	= nil
		
		@@use_win32_native = false
	
		def initialize(owner, name, options = 0)
			super(owner, name, options)
			
			self.selectMode = SELECTFILE_EXISTING
			
			if $".include?("Win32API.so") and @@get_file_name == nil
				begin
					@@get_file_name = Win32API.new('comdlg32', 'GetOpenFileName', ['P'], 'i')
					@@get_save_name = Win32API.new('comdlg32', 'GetSaveFileName', ['P'], 'i')
					@@get_active_window = Win32API.new("user32", "GetActiveWindow", [], 'L')
					
					@@use_win32_native = true
				rescue
					@@get_file_name = @@get_save_name = @@get_active_window = -1
				end
			end
		end
		
		
		def getFilenames
			return @filenames if @filenames
			super
		end
		
		
		def execute(placement = PLACEMENT_SCREEN)
			if @@use_win32_native
				if (selectMode & SELECTFILE_EXISTING) > 0
					get_file_name = @@get_file_name
				else
					get_file_name = @@get_save_name
				end
				
				pattern_string = ""
				
				if patternList
					patternList.sort.each { |p|
						p =~ /\(.*\)/
						pattern_string += "#{$`} #{$&}\0#{$&[1..-2].gsub(',', ';')}\0"
					}
				end
				
				pattern_string += "\0"
				
				files = "#{self.filename}" + "\0" * (1024 - self.filename.size)

				parameters_array = [ 0 ] * 20
				
				parameters_array[0] 	= 76 # size of struct
				parameters_array[1]		= @@get_active_window.Call # owner
				parameters_array[3] 	= pattern_string	# filter
				parameters_array[6]		= currentPattern + 1 # initially selected pattern
				parameters_array[7]		= files # initial filename and returned filename buffer
				parameters_array[8]		= files.size # max size of returned filenames
				parameters_array[11]	= directory # initial directory
				parameters_array[12]	= title # window title
				parameters_array[13] 	= 0x4 # flags (no read only button)
				parameters_array[13]	+= 0x1000 if (selectMode & SELECTFILE_EXISTING) > 0
				parameters_array[13]	+= 0x200 + 0x80000 if (selectMode & SELECTFILE_MULTIPLE) > 0 # 0x80000 is explorer mode
				
				parameters = parameters_array.pack('lllPlllPlllppliillll')
				
				if get_file_name.Call(parameters) == 1
					if (selectMode & SELECTFILE_MULTIPLE) > 0
						@filenames = files.split("\0")
					
						if @filenames.size > 1
							@filenames.each_with_index { |f, i|
								@filenames[i] = "#{@filenames[0]}#{File::Separator}#{f}" unless i == 0
							}

							@filenames.shift
						else
							self.filename = @filenames[0]
						end
					else
						self.filename = files.split("\0")[0]
					end
					
					return 1
				else
					return 0
				end
			else
				return super
			end
		end
	end



	class FileChooser < FXTextField
		def initialize(parent, title, cols, tgt = nil, sel = 0, main_opts = FRAME_NONE, sub_opts = FRAME_NONE, field_opts = FRAME_LINE)
			frame = FXHorizontalFrame.new(parent, main_opts)
			
			FXLabel.new(frame, title, nil, LAYOUT_CENTER_Y)
			
			frame = FXHorizontalFrame.new(frame, sub_opts | LAYOUT_CENTER_Y, 0, 0, 0, 0, 0, 0, 0, 0)
			
			super(frame, cols, tgt, sel, field_opts | LAYOUT_CENTER_Y)
			
			FXButton.new(frame, "Browse...", nil, nil, 0, BUTTON_NORMAL | LAYOUT_CENTER_Y).connect(SEL_COMMAND) {
        dlg = FXFileDialog.new(self, "Choose File")
        if dlg.execute == 1
          self.text = dlg.filename
					target.handle(dlg, MKUINT(self.selector, SEL_CHANGED), self.text) if target
        end
			}
		end
	end	

	
	
	class DirChooser < FXTextField
		def initialize(parent, title, cols, tgt = nil, sel = 0, main_opts = FRAME_NONE, sub_opts = FRAME_NONE, field_opts = FRAME_LINE)
			frame = FXHorizontalFrame.new(parent, main_opts)
			
			FXLabel.new(frame, title, nil, LAYOUT_CENTER_Y)
			
			frame = FXHorizontalFrame.new(frame, sub_opts | LAYOUT_CENTER_Y, 0, 0, 0, 0, 0, 0, 0, 0)
			
			super(frame, cols, tgt, sel, field_opts | LAYOUT_CENTER_Y)
			
			FXButton.new(frame, "Browse...", nil, nil, 0, BUTTON_NORMAL | LAYOUT_CENTER_Y).connect(SEL_COMMAND) {
        dlg = FXDirDialog.new(self, "Choose File")
        if dlg.execute == 1
          self.text = dlg.directory
					target.handle(dlg, MKUINT(self.selector, SEL_CHANGED), self.text) if target
        end
			}
		end
	end	
	
	
	
	class GenericFile
		alias_method :_close, 	:close
		alias_method :_rename, 	:rename
		
		def rename(filename = nil, change_file = false)
			unless filename
				dlg = FXDialogBox.new(SessionManager.instance.main_window, "Rename File", DECOR_TITLE | DECOR_BORDER, 0, 0, 0, 0)
				
				frame = FXHorizontalFrame.new(dlg, FRAME_NONE)
				
				FXLabel.new(frame, "Filename:")
				
				nameField = FXTextField.new(frame, 25, dlg, FXDialogBox::ID_ACCEPT, TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY)
				nameField.text = self.name
				nameField.setFocus
				
				frame = FXHorizontalFrame.new(dlg, FRAME_NONE | LAYOUT_SIDE_RIGHT)
				
				button = FXButton.new(frame, "OK", nil, dlg, FXDialogBox::ID_ACCEPT, BUTTON_NORMAL | LAYOUT_FIX_WIDTH)
				button.resize(button.font.getTextWidth("Cancel") + 25, 0)
				
				button = FXButton.new(frame, "Cancel", nil, dlg, FXDialogBox::ID_CANCEL, BUTTON_NORMAL | LAYOUT_FIX_WIDTH)
				button.resize(button.font.getTextWidth("Cancel") + 25, 0)
				
				if dlg.execute(PLACEMENT_OWNER) == 1
					filename = self.directory + File::Separator + nameField.text
					change_file = true
				else
					return false
				end
			end
			
			begin
				_rename(filename, change_file)
			rescue
				FXMessageBox.error(SessionManager.instance.main_window, MBOX_OK, "Rename File", "Could not change filename to '#{File.basename(filename)}'.\n\n#{$!.message}")
				return false
			end
			
			true
		end
		
		
		def close
			if dirty?
				answer = FXMessageBox.question(SessionManager.instance.main_window, MBOX_YES_NO_CANCEL, "Close File", "File '#{full_name}' has changed\nsince you last saved.  Do you want to save the changes?")
				
				return false if answer == MBOX_CLICKED_CANCEL
				
				if answer == MBOX_CLICKED_YES
					return false unless save
				end
			end
			
			_close
			
			true
		end
	end
	
	
	
	class RubyFile
		alias_method :_save, 				:save
		alias_method :_parameters=, :parameters=
		
		def run
			if dirty?
				return unless (SessionManager.instance.restore("Preferences/Auto Save") or 
												FXMessageBox.question(SessionManager.instance.main_window, MBOX_OK_CANCEL, "Run File", "File '#{self.full_name}' must be\nsaved before execution.  Save it now?") == MBOX_CLICKED_OK) and
											save
			end
			
			ExecutionManager.instance.execute(self)
		end
		
		
		def save_as
      saveDlg = FXFileDialog.new(SessionManager.instance.main_window, "Save File As")
      
      saveDlg.selectMode      = SELECTFILE_ANY
      saveDlg.patternList     = ["All Files (*)", "Ruby Files (*.rb,*.rbw)"]
      saveDlg.currentPattern  = 1
			saveDlg.filename				= self.name
      
      if saveDlg.execute(PLACEMENT_OWNER) != 0
				if File.exist?(File.expand_path(saveDlg.filename))
					if FileManager.instance.file_opened?(File.expand_path(saveDlg.filename))
						FXMessageBox.error(SessionManager.instance.main_window, MBOX_OK, "Save File As", "File is currently opened.  Cannot overwrite.")
						return false
					end
					
					if FXMessageBox.question(SessionManager.instance.main_window, MBOX_YES_NO, "Save File As", "File exists.  Overwrite?") == MBOX_CLICKED_YES
						File.delete(File.expand_path(saveDlg.filename))
					else
						return false
					end
				end
				
				oldname 	= @filename					
				@filename = File.expand_path(saveDlg.filename)
				
				begin
					_save
					
					notify(FILE_RENAMED, oldname)
				rescue
					FXMessageBox.error(SessionManager.instance.main_window, MBOX_OK, "Save File As", $!.message)
					
					@filename = oldname
					
					return false
				end
				
				return true
      end
			
			false
		end
		
		
		def save
			if is_new?
				return save_as
			else
				begin
					_save
				rescue
					FXMessageBox.error(SessionManager.instance.main_window, MBOX_OK, "Save File", $!.message)
					
					return false
				end
				
				return true
			end
		end
		
		
		def edit_environment
			dlg = FXDialogBox.new(SessionManager.instance.main_window, "#{name} Environment", DECOR_TITLE | DECOR_BORDER, 0, 0, 0, 0)
			
			frame = FXGroupBox.new(dlg, "Execution Environment", GROUPBOX_NORMAL | FRAME_THICK)
			
			startup_dir = DirChooser.new(frame, "Execution directory:", 35, dlg, FXDialogBox::ID_ACCEPT, LAYOUT_FILL_X, LAYOUT_RIGHT, TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY)
			startup_dir.text = properties["Execution Directory"] unless properties["Execution Directory"] == nil
			
			pre_exec = FileChooser.new(frame, "Pre-execution command:", 35, dlg, FXDialogBox::ID_ACCEPT, LAYOUT_FILL_X, LAYOUT_RIGHT, TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY)
			pre_exec.text = properties["Pre-execution Command"] unless properties["Pre-execution Command"] == nil

			post_exec = FileChooser.new(frame, "Post-execution command:", 35, dlg, FXDialogBox::ID_ACCEPT, LAYOUT_FILL_X, LAYOUT_RIGHT, TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY)
			post_exec.text = properties["Post-execution Command"] unless properties["Post-execution Command"] == nil

			frame = FXHorizontalFrame.new(dlg, FRAME_NONE | LAYOUT_SIDE_RIGHT)
			
			button = FXButton.new(frame, "OK", nil, dlg, FXDialogBox::ID_ACCEPT, BUTTON_NORMAL | LAYOUT_FIX_WIDTH)
			button.resize(button.font.getTextWidth("Cancel") + 25, 0)
			
			button = FXButton.new(frame, "Cancel", nil, dlg, FXDialogBox::ID_CANCEL, BUTTON_NORMAL | LAYOUT_FIX_WIDTH)
			button.resize(button.font.getTextWidth("Cancel") + 25, 0)
			
			if dlg.execute(PLACEMENT_OWNER) == 1
				properties["Execution Directory"] 		= File.expand_path(startup_dir.text)
				properties["Execution Directory"]			= nil if startup_dir.text == ""
				properties["Pre-execution Command"] 	= pre_exec.text
				properties["Pre-execution Command"]		= nil if pre_exec.text == ""
				properties["Post-execution Command"]	= post_exec.text
				properties["Post-execution Command"]	= nil if post_exec.text == ""
				
				notify(FILE_PROPERTIES_CHANGED)
			else
				return false
			end
		end


		def populate_menu(menu)
			FXMenuCommand.new(menu, "Save File").connect(SEL_COMMAND) { self.save }
			command = FXMenuCommand.new(menu, "Run File")
			command.connect(SEL_COMMAND) { self.run }
			command.disable if ExecutionManager.instance.is_running?
			FXMenuSeparator.new(menu)
			FXMenuCommand.new(menu, "Rename File...").connect(SEL_COMMAND) { self.rename }
			FXMenuCommand.new(menu, "Edit Environment...").connect(SEL_COMMAND) { self.edit_environment }
			FXMenuCommand.new(menu, "Edit Parameters...").connect(SEL_COMMAND) { self.edit_parameters }
			FXMenuSeparator.new(menu)
			project = SessionManager.instance.get_associated_project(self)
			if project
				command = FXMenuCheck.new(menu, "Set As Project Main")
				command.connect(SEL_COMMAND) { project.project_key = self }
				command.check = true if project.is_project_key?(self)
				FXMenuCommand.new(menu, "Remove From Project").connect(SEL_COMMAND) { project.remove_file(self) }
			else
				command = FXMenuCommand.new(menu, "Add To Active Project")
				command.connect(SEL_COMMAND) { SessionManager.instance.active_project.add_file(self) }
				command.disable unless SessionManager.instance.active_project and !self.is_new?
				FXMenuCommand.new(menu, "Remove From Workspace").connect(SEL_COMMAND) { self.close }
			end
		end
		
		
		def edit_parameters
			dlg = FXDialogBox.new(SessionManager.instance.main_window, "#{name} Parameters", DECOR_TITLE | DECOR_BORDER, 0, 0, 0, 0)
      
			group_box = FXGroupBox.new(dlg, "Command-line Parameters", GROUPBOX_NORMAL | FRAME_THICK | LAYOUT_FILL_X)
			
			parameter_fields = []
			
			5.times do |i|
				frame = FXHorizontalFrame.new(group_box, FRAME_NONE, 0, 0, 0, 0)
				FXLabel.new(frame, "Parameter #{i + 1}:")
				field = FXTextField.new(frame, 25, dlg, FXDialogBox::ID_ACCEPT, TEXTFIELD_NORMAL | LAYOUT_RIGHT | TEXTFIELD_ENTER_ONLY)
				field.text = self.parameters[i]
				parameter_fields << field
			end

			frame = FXHorizontalFrame.new(dlg, FRAME_NONE | LAYOUT_SIDE_RIGHT, 0, 0, 0, 0)
			button = FXButton.new(frame, "OK", nil, dlg, FXDialogBox::ID_ACCEPT, BUTTON_NORMAL | LAYOUT_FIX_WIDTH)
			button.resize(button.font.getTextWidth("Cancel") + 25, 0)
			button = FXButton.new(frame, "Cancel", nil, dlg, FXDialogBox::ID_CANCEL, BUTTON_NORMAL | LAYOUT_FIX_WIDTH)
			button.resize(button.font.getTextWidth("Cancel") + 25, 0)
			
			parameter_fields[0].setFocus
				
			if dlg.execute(PLACEMENT_OWNER) == 1
				parameters = []
				
				parameter_fields.reverse.each { |field|
					parameter = field.text
					parameter.gsub!(/(\A\")|(\"\Z)/, "")
					parameters[0, 0] = "\"" + parameter + "\"" if parameter != "" or parameters.size > 0
				}
				
				self.parameters = parameters
			end
		end
	end



	class ProjectFile
		alias_method :_save, :save
		
		def save
			begin
				_save
			rescue
				FXMessageBox.error(SessionManager.instance.main_window, MBOX_OK, "Save Project", $!.message)
			end
		end
		
	
		def run
			if @project_key
				auto_save = SessionManager.instance.restore("Preferences/Auto Save")
				
				each_file { |file| 
					if file.dirty?
						unless auto_save
							return unless FXMessageBox.question(SessionManager.instance.main_window, MBOX_OK_CANCEL, "Run Project", "All files in '#{project_name}' must be saved before\nexecution.  Save them now?") == MBOX_CLICKED_OK and
														file.save
							auto_save = true
						else
							return unless file.save
						end
					end
				}
				
				ExecutionManager.instance.execute(FileManager.instance.open_file(self.project_key))
			else
        FXMessageBox.information(SessionManager.instance.main_window, MBOX_OK, "Please select a project main file.", "No project main file is set for this project.  The project main file is the file that\nwill be executed when you run the project.  To set a project main file,\nright-click on a file and select \"Set As Project Main\", or use the\n\"Project->Set Current File As Project Main\" menu option.")
			end
		end
		
		
		def close
			each_filename { |filename|
				begin
					file = FileManager.instance.open_file(filename) if FileManager.instance.file_opened?(filename)
				rescue
				end
				if file and file.dirty?
					answer = FXMessageBox.question(SessionManager.instance.main_window, MBOX_YES_NO_CANCEL, "Close Project", "File '#{file.full_name}' has not been\nsaved.  Do you want to save the changes?")
					begin
						return false if answer == MBOX_CLICKED_CANCEL or (answer == MBOX_CLICKED_YES and !file.save)
					rescue
						FXMessageBox.error(SessionManager.instance.main_window, MBOX_OK, "Close Project", $!.message)
						return false unless file.save_as
					end
				end
			}
			
			each_filename { |filename|
				begin
					file = FileManager.instance.open_file(filename) if FileManager.instance.file_opened?(filename)
					file._close if file
				rescue
					FXMessageBox.error(SessionManager.instance.main_window, MBOX_OK, "Close Project", $!.message)
					return false unless file.save_as
				end
			}
			
			_close
			
			GC::start
			
			true
		end
		
		
		def add_files
			openDlg = FXFileDialog.new(SessionManager.instance.main_window, "Add File(s)")
			
			openDlg.selectMode      = SELECTFILE_MULTIPLE | SELECTFILE_EXISTING
			openDlg.patternList     = ["All Files (*)", "Ruby Files (*.rb,*.rbw)"]
			openDlg.currentPattern  = 1
 
			if openDlg.execute(PLACEMENT_OWNER) != 0
				openDlg.getFilenames.each do |filename|
					begin
						add_file(filename)
					rescue
						FXMessageBox.error(SessionManager.instance.main_window, MBOX_OK, "Add Files", $!.message)
					end
				end
			end
		end
		
		
		def search(text)
			SearchFilesDialog.instance.execute(self)
		end
		
		
		def project_settings
			dlg = FXDialogBox.new(SessionManager.instance.main_window, "#{project_name} Settings", DECOR_TITLE | DECOR_BORDER, 0, 0, 0, 0)
			
			frame = FXGroupBox.new(dlg, "Project Statistics", GROUPBOX_NORMAL | FRAME_THICK | LAYOUT_FILL_X)
			
			FXLabel.new(frame, "Files: #{@files.size}")
			
			lines = 0
			begin
				each_file do |file| 
					lines += file.lines 
				end
			rescue
			end
			FXLabel.new(frame, "Lines: #{lines}")
			
			frame = FXGroupBox.new(dlg, "Execution Environment", GROUPBOX_NORMAL | FRAME_THICK)
			
			startup_dir = DirChooser.new(frame, "Execution directory:", 35, dlg, FXDialogBox::ID_ACCEPT, LAYOUT_FILL_X, LAYOUT_RIGHT, TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY)
			startup_dir.text = properties["Execution Directory"] unless properties["Execution Directory"] == nil
			
			pre_exec = FileChooser.new(frame, "Pre-execution command:", 35, dlg, FXDialogBox::ID_ACCEPT, LAYOUT_FILL_X, LAYOUT_RIGHT, TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY)
			pre_exec.text = properties["Pre-execution Command"] unless properties["Pre-execution Command"] == nil

			post_exec = FileChooser.new(frame, "Post-execution command:", 35, dlg, FXDialogBox::ID_ACCEPT, LAYOUT_FILL_X, LAYOUT_RIGHT, TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY)
			post_exec.text = properties["Post-execution Command"] unless properties["Post-execution Command"] == nil

			frame = FXHorizontalFrame.new(dlg, FRAME_NONE | LAYOUT_SIDE_RIGHT)
			
			button = FXButton.new(frame, "OK", nil, dlg, FXDialogBox::ID_ACCEPT, BUTTON_NORMAL | LAYOUT_FIX_WIDTH)
			button.resize(button.font.getTextWidth("Cancel") + 25, 0)
			
			button = FXButton.new(frame, "Cancel", nil, dlg, FXDialogBox::ID_CANCEL, BUTTON_NORMAL | LAYOUT_FIX_WIDTH)
			button.resize(button.font.getTextWidth("Cancel") + 25, 0)
			
			if dlg.execute(PLACEMENT_OWNER) == 1
				properties["Execution Directory"] 		= File.expand_path(startup_dir.text)
				properties["Execution Directory"]			= nil if startup_dir.text == ""
				properties["Pre-execution Command"] 	= pre_exec.text
				properties["Pre-execution Command"]		= nil if pre_exec.text == ""
				properties["Post-execution Command"]	= post_exec.text
				properties["Post-execution Command"]	= nil if post_exec.text == ""
				
				save
			else
				return false
			end
		end
		
		
		def populate_menu(menu)
			command = FXMenuCommand.new(menu, "Run Project")
			command.connect(SEL_COMMAND) { self.run }
			command.disable if ExecutionManager.instance.is_running?
			FXMenuSeparator.new(menu)
			FXMenuCommand.new(menu, "New File").connect(SEL_COMMAND) { self.new_file }
			FXMenuCommand.new(menu, "Add Files...").connect(SEL_COMMAND) { self.add_files }
			FXMenuSeparator.new(menu)
			FXMenuCommand.new(menu, "Project Settings...").connect(SEL_COMMAND) { self.project_settings }
			command = FXMenuCheck.new(menu, "Set As Active Project")
			command.connect(SEL_COMMAND) { SessionManager.instance.set_active_project(self) }
			command.check = true if SessionManager.instance.active_project == self
			FXMenuSeparator.new(menu)
			FXMenuCommand.new(menu, "Close Project").connect(SEL_COMMAND) { self.close }
		end
		
		
		def new_file
			add_file(FileManager.instance.create_new_file_in_directory(RubyFile, self.directory))
		end
	end

end