# Purpose:  Launch the main window
#
# Notes:	The CMD/invoke scheme really could be object-based.
#

$: << File.expand_path(File.dirname(__FILE__)) # add Mondrian's directory to the module search path

require 'fox12'

require 'SessionManager'
require 'IconManager'
require 'FileManager'
require 'ExecutionManager'
require 'GuiExtensions.rb'
require 'PropertiesFrame'
require 'EditBook'
require 'DockFrame'
require 'SourceBrowser'
require 'ExecutionMonitor'
require 'HomeViewer'
require 'ObjectGarden'
require 'CreateProjectDialog'
require 'PreferencesDialog'
require 'SearchFrame'

include Fox


# fix weirdness in Fox 1.2's FXMenuCheck
class FXMenuCheck
	def checked?
		return self.getCheck != TRUE
	end
end


# fix weirdness in Fox 1.2's FXCheckButton
class FXCheckButton
	def checked?
		return self.getCheck
	end
end


module Mondrian
  # command capabilities
	CMD_FILE						= 0
  CMD_UNDO        		= 1
  CMD_REDO        		= 2
  CMD_CUT         		= 3
  CMD_COPY        		= 4
  CMD_PASTE       		= 5
  CMD_DELETE      		= 6
  CMD_FIND        		= 7
  CMD_FIND_NEXT    		= 8
  CMD_FIND_PREVIOUS		= 9
	CMD_REPLACE					= 10
	CMD_REPLACE_ALL			= 11
  CMD_GOTO_LINE   		= 12
  CMD_SELECT_ALL  		= 13
	CMD_BLOCK_COMMENT		= 14
	CMD_BACK						= 15
	CMD_FORWARD					= 16
	
	# GUI features expressed as capabilities
	GUI_CURSOR_POSITION	= 50
	GUI_QUICK_FIND			= 51
	

	#
	# Mondrian's main window
	#
	
  class MainWindow < FXMainWindow
    def initialize(app)
      icon = FXBMPIcon.new(app, nil, FXRGB(255, 255, 255), IMAGE_ALPHACOLOR)
      FXFileStream.open(PROGRAM_DIRECTORY + File::Separator + "images/mondrian_icon.bmp", FXStreamLoad) do |stream| 
        icon.loadPixels(stream)
      end
      icon.create

      small_icon = FXBMPIcon.new(app, nil, FXRGB(255, 255, 255), IMAGE_ALPHACOLOR)
      FXFileStream.open(PROGRAM_DIRECTORY + File::Separator + "images/mondrian_icon_small.bmp", FXStreamLoad) do |stream| 
        small_icon.loadPixels(stream)
      end
      small_icon.create
			
      super(app, "Mondrian Ruby IDE", icon, small_icon, DECOR_ALL, 0, 0, 800, 600)
      
      FileManager.instance.add_observer(self)
      ExecutionManager.instance.add_observer(self)
      SessionManager.instance.add_observer(self)
			
			SessionManager.instance.main_window = self

      register_filetypes
      register_icons

      #
      # basic layout
      #
      
			FXToolTip.new(app) # initialize tooltips
		
      @menubar = FXMenuBar.new(self, LAYOUT_SIDE_TOP | LAYOUT_FILL_X, 0, 0, 0, 0, 3, 3, 0, 0, 0, 0)
      
      FXHorizontalSeparator.new(self, LAYOUT_FILL_X | LAYOUT_SIDE_TOP | SEPARATOR_GROOVE)
      
      @toolbar = FXToolBar.new(self, LAYOUT_SIDE_TOP | LAYOUT_FILL_X | LAYOUT_FIX_HEIGHT, 0, 0, 0, 25, 4, 4, 0, 0, 0, 0)
      
      @statusbar = FXStatusBar.new(self, LAYOUT_SIDE_BOTTOM | LAYOUT_FILL_X, 0, 0, 0, 0, 0, 0, 0, 0)
			
      vSplitter = FXSplitter.new(self, LAYOUT_FILL_X | LAYOUT_FILL_Y | SPLITTER_TRACKING | SPLITTER_VERTICAL | SPLITTER_REVERSED)
      hSplitter = FXSplitter.new(vSplitter, LAYOUT_FILL_X | LAYOUT_FILL_Y | LAYOUT_FIX_HEIGHT | SPLITTER_TRACKING | SPLITTER_HORIZONTAL, 0, 0, 0, 475)
      
			@dock_left = DockFrame.new(hSplitter, DOCKFRAME_BOTTOMTABS | LAYOUT_FIX_WIDTH | LAYOUT_FILL_X | LAYOUT_FILL_Y, 0, 0, 200, 0, 0, 0, 0, 0)
      @dock_left.connect(SEL_UNMAP,  method(:onSidebarChange))
      @dock_left.connect(SEL_MAP,    method(:onSidebarChange))

      @sourceBrowser = SourceBrowser.new(@dock_left)
			
      # we could add more frames to the first splitter, but adding another makes resizing more friendly
      hSplitter = FXSplitter.new(hSplitter, LAYOUT_FILL_X | LAYOUT_FILL_Y | LAYOUT_FIX_HEIGHT | SPLITTER_TRACKING | SPLITTER_HORIZONTAL | SPLITTER_REVERSED, 0, 0, 0, 475)
      
      @editFrame = FXVerticalFrame.new(hSplitter, FRAME_NONE | LAYOUT_FILL_X | LAYOUT_FILL_Y, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
      
      @editBook = EditBook.new(@editFrame, FRAME_THICK | LAYOUT_FILL_X | LAYOUT_FILL_Y, 0, 0, 0, 0, 0, 0, 0, 0)

			@dock_right = DockFrame.new(hSplitter, DOCKFRAME_BOTTOMTABS | LAYOUT_FILL_X | LAYOUT_FILL_Y | LAYOUT_FIX_WIDTH, 0, 0, 150, 0, 0, 0, 0, 0)
      @dock_right.connect(SEL_UNMAP,  method(:onSidebarChange))
      @dock_right.connect(SEL_MAP,    method(:onSidebarChange))
      
			@propFrame = PropertiesFrame.new(@dock_right)

			@dock_bottom = DockFrame.new(vSplitter, DOCKFRAME_BOTTOMTABS | LAYOUT_FILL_X | LAYOUT_FILL_Y | LAYOUT_FIX_HEIGHT, 0, 0, 0, 100, 0, 0, 0, 0)
      @dock_bottom.connect(SEL_UNMAP,  method(:onSidebarChange))
      @dock_bottom.connect(SEL_MAP,    method(:onSidebarChange))
			
      @reportFrame = ExecutionMonitor.new(@dock_bottom)
			@searchFrame = SearchFrame.new(@dock_bottom)

      self.connect(SEL_CLOSE, method(:onQuit))
      
      # initialize variables
      
      @gui            = {}  # a registry of gui objects such as buttons and menu items
      @focused_viewer = nil # the currently active viewer in the edit book
      
      populate_menu      
      populate_toolbar
			populate_statusbar
    end

		
    def register_filetypes
      fileManager = FileManager.instance
      
      fileManager.register_filetype(".mpr", ProjectFile)
      fileManager.register_filetype(".rb",  RubyFile)
      fileManager.register_filetype(".rbw", RubyFile)
			
			fileManager.default_filetype = RubyFile
    end
    
    
    def register_icons
      iconManager = IconManager.instance
      
      iconManager.register_icon(GenericFile,  IconManager.load_icon(getApp, "file.png"))
      iconManager.register_icon(ProjectFile,  IconManager.load_icon(getApp, "project.png"))
      iconManager.register_icon(RubyFile,     IconManager.load_icon(getApp, "file.png"))
      iconManager.register_icon(RubyModule,   IconManager.load_icon(getApp, "module.png"))
      iconManager.register_icon(RubyClass,    IconManager.load_icon(getApp, "class.png"))
      iconManager.register_icon(RubyMethod,   IconManager.load_icon(getApp, "method.png"))
    end


		def register_default_viewers
			@editBook.register_default_viewer(ProjectFile,  ObjectGarden::ObjectGarden)
			@editBook.register_default_viewer(RubyFile, 		RubyText)
			@editBook.register_default_viewer(RubyModule,		RubyText)
			@editBook.register_default_viewer(RubyClass, 		RubyText)
			@editBook.register_default_viewer(RubyMethod, 	RubyText)
		end
		
    
    def populate_toolbar
      @gui["toolbar.New File"]  = FXButton.new(@toolbar, "\tNew File\tCreate a new file.", IconManager.load_icon(getApp, "new.png"), nil, 0, BUTTON_TOOLBAR | BUTTON_NORMAL)      
      @gui["toolbar.New File"].connect(SEL_COMMAND, method(:onNewFile))
      
      @gui["toolbar.Open File"] = FXButton.new(@toolbar, "\tOpen File\tOpen an existing file.", IconManager.load_icon(getApp, "open.png"), nil, 0, BUTTON_TOOLBAR | BUTTON_NORMAL)
      @gui["toolbar.Open File"].connect(SEL_COMMAND, method(:onOpenFile))
      
      @gui["toolbar.Close File"] = FXButton.new(@toolbar, "\tClose File\tClose the current file or editor tab.", IconManager.load_icon(getApp, "close.png"), nil, 0, BUTTON_TOOLBAR | BUTTON_NORMAL)
      @gui["toolbar.Close File"].connect(SEL_COMMAND, method(:onCloseFile))
      
      @gui["toolbar.Save File"] = FXButton.new(@toolbar, "\tSave File\tSave the current editor tab.", IconManager.load_icon(getApp, "save.png"), nil, 0, BUTTON_TOOLBAR | BUTTON_NORMAL)
      @gui["toolbar.Save File"].connect(SEL_COMMAND, method(:onSave))

      FXVerticalSeparator.new(@toolbar, LAYOUT_FIX_HEIGHT | LAYOUT_SIDE_LEFT | SEPARATOR_GROOVE, 0, 0, 0, 25, 6, 6)
			
      @gui["toolbar.Back"] = FXButton.new(@toolbar, "\tBack\tBack to the last editing position.", IconManager.load_icon(getApp, "back.png"), nil, 0, BUTTON_TOOLBAR | BUTTON_NORMAL)
      @gui["toolbar.Back"].connect(SEL_COMMAND, method(:onBack))
      
      @gui["toolbar.Forward"] = FXButton.new(@toolbar, "\tForward\tForward to the next editing position.", IconManager.load_icon(getApp, "forward.png"), nil, 0, BUTTON_TOOLBAR | BUTTON_NORMAL)
      @gui["toolbar.Forward"].connect(SEL_COMMAND, method(:onForward))
			
      FXVerticalSeparator.new(@toolbar, LAYOUT_FIX_HEIGHT | LAYOUT_SIDE_LEFT | SEPARATOR_GROOVE, 0, 0, 0, 25, 6, 6)
      
			@gui["toolbar.Undo"] = FXButton.new(@toolbar, "\tUndo\tUndo the last operation.", IconManager.load_icon(getApp, "undo.png"), nil, 0, BUTTON_TOOLBAR | BUTTON_NORMAL)
      @gui["toolbar.Undo"].connect(SEL_COMMAND, method(:onUndo))
      
      @gui["toolbar.Redo"] = FXButton.new(@toolbar, "\tRedo\tRedo the last undone operation.", IconManager.load_icon(getApp, "redo.png"), nil, 0, BUTTON_TOOLBAR | BUTTON_NORMAL)
      @gui["toolbar.Redo"].connect(SEL_COMMAND, method(:onRedo))
			
      FXVerticalSeparator.new(@toolbar, LAYOUT_FIX_HEIGHT | LAYOUT_SIDE_LEFT | SEPARATOR_GROOVE, 0, 0, 0, 25, 6, 6)
			
      @gui["toolbar.Cut"] = FXButton.new(@toolbar, "\tCut\tCut the current selection.", IconManager.load_icon(getApp, "cut.png"), nil, 0, BUTTON_TOOLBAR | BUTTON_NORMAL)
      @gui["toolbar.Cut"].connect(SEL_COMMAND, method(:onCut))
			
      @gui["toolbar.Copy"] = FXButton.new(@toolbar, "\tCopy\tCopy the current selection to the clipboard.", IconManager.load_icon(getApp, "copy.png"), nil, 0, BUTTON_TOOLBAR | BUTTON_NORMAL)
      @gui["toolbar.Copy"].connect(SEL_COMMAND, method(:onCopy))
			
      @gui["toolbar.Paste"] = FXButton.new(@toolbar, "\tCopy\tPaste the contents of the clipboard.", IconManager.load_icon(getApp, "paste.png"), nil, 0, BUTTON_TOOLBAR | BUTTON_NORMAL)
      @gui["toolbar.Paste"].connect(SEL_COMMAND, method(:onPaste))
			
      FXVerticalSeparator.new(@toolbar, LAYOUT_FIX_HEIGHT | LAYOUT_SIDE_LEFT | SEPARATOR_GROOVE, 0, 0, 0, 25, 6, 6)
        
			frame = FXHorizontalFrame.new(@toolbar, 0, 0, 0, 0, 0, 6, 6, 0, 0) # combobox doesn't space correctly
      @gui["toolbar.Quick Find"] = FXComboBox.new(frame, 20, nil, 0, COMBOBOX_NO_REPLACE | FRAME_LINE)
			@gui["toolbar.Quick Find"].setNumVisible(10)
			@gui["toolbar.Quick Find"].connect(SEL_COMMAND, method(:onQuickFind))
			
      @gui["toolbar.Find..."] = FXButton.new(@toolbar, "\tFind\tSearch the current editor tab.", IconManager.load_icon(getApp, "find.png"), nil, 0, BUTTON_TOOLBAR | BUTTON_NORMAL)
      @gui["toolbar.Find..."].connect(SEL_COMMAND, method(:onFind))
			
      @gui["toolbar.Search Files..."] = FXButton.new(@toolbar, "\tSearch Files\tSearch for text in multiple files.", IconManager.load_icon(getApp, "find_in_project.png"), nil, 0, BUTTON_TOOLBAR | BUTTON_NORMAL)
      @gui["toolbar.Search Files..."].connect(SEL_COMMAND, method(:onSearchFiles))
			
      FXVerticalSeparator.new(@toolbar, LAYOUT_FIX_HEIGHT | LAYOUT_SIDE_LEFT | SEPARATOR_GROOVE, 0, 0, 0, 25, 6, 6)

      @gui["toolbar.Run File"] = FXButton.new(@toolbar, "\tRun File\tRun the active file.", IconManager.load_icon(getApp, "run.png"), nil, 0, BUTTON_TOOLBAR | BUTTON_NORMAL)
      @gui["toolbar.Run File"].connect(SEL_COMMAND, method(:onRunFile))
			
      @gui["toolbar.Run Project"] = FXButton.new(@toolbar, "\tRun Project\tRun the active project.", IconManager.load_icon(getApp, "run_project.png"), nil, 0, BUTTON_TOOLBAR | BUTTON_NORMAL)
      @gui["toolbar.Run Project"].connect(SEL_COMMAND, method(:onRunProject))
    end
		
		
		def populate_statusbar
			frame = FXHorizontalFrame.new(@statusbar, FRAME_SUNKEN, 0, 0, 0, 0, 0, 0, 0, 0)
			
			@gui["statusbar.Line Label"] 		= FXLabel.new(frame, "Line:")
			@gui["statusbar.Line"] 					= FXLabel.new(frame, "")
			@gui["statusbar.Column Label"] 	= FXLabel.new(frame, "Column:")
			@gui["statusbar.Column"] 				= FXLabel.new(frame, "")
			
			FXDragCorner.new(frame)
		end
    
    
    def populate_menu
      # file menu
      
      pane = FXMenuPane.new(self)
      
      @gui["menu.File/New File"]        = FXMenuCommand.new(pane, "&New File\tCtl-N\tCreate a new file.", IconManager.load_icon(getApp, "new.png"))
      @gui["menu.File/New File"].connect(SEL_COMMAND, method(:onNewFile))
      @gui["menu.File/Open File..."]    = FXMenuCommand.new(pane, "&Open File...\tCtl-O\tOpen an existing file.", IconManager.load_icon(getApp, "open.png"))
      @gui["menu.File/Open File..."].connect(SEL_COMMAND, method(:onOpenFile))
      @gui["menu.File/Save File"]       = FXMenuCommand.new(pane, "&Save File\tCtl-S\tSave the current editor tab.", IconManager.load_icon(getApp, "save.png"))
      @gui["menu.File/Save File"].connect(SEL_COMMAND, method(:onSave))
      @gui["menu.File/Save File As..."] = FXMenuCommand.new(pane, "Save File &As...\t\tSave and name the current file.")
      @gui["menu.File/Save File As..."].connect(SEL_COMMAND, method(:onSaveAs))
      @gui["menu.File/Close File"] = FXMenuCommand.new(pane, "&Close File\tCtl-W\tClose the current file or editor tab.", IconManager.load_icon(getApp, "close.png"))
      @gui["menu.File/Close File"].connect(SEL_COMMAND, method(:onCloseFile))
      @gui["menu.File/Remove File From Workspace"] = FXMenuCommand.new(pane, "&Remove File From Workspace\tCtl-R\tRemove the current file from the workspace.")
      @gui["menu.File/Remove File From Workspace"].connect(SEL_COMMAND, method(:onRemoveFileFromWorkspace))
      
      FXMenuSeparator.new(pane)
    
      @gui["menu.File/New Project..."]  = FXMenuCommand.new(pane, "New Project...\t\tCreate a new project.")
      @gui["menu.File/New Project..."].connect(SEL_COMMAND, method(:onNewProject))
      @gui["menu.File/Open Project..."] = FXMenuCommand.new(pane, "Open &Project...\t\tOpen an existing project.")
      @gui["menu.File/Open Project..."].connect(SEL_COMMAND, method(:onOpenProject))
      @gui["menu.File/Close Project"]   = FXMenuCommand.new(pane, "Close Project\t\tClose the active project.")
			@gui["menu.File/Close Project"].connect(SEL_COMMAND, method(:onCloseProject))
      
      FXMenuSeparator.new(pane)
      
      @gui["menu.File/Save All"] = FXMenuCommand.new(pane, "Save All\t\tSave all open files.")
			@gui["menu.File/Save All"].connect(SEL_COMMAND, method(:onSaveAll))
      
      FXMenuSeparator.new(pane)
      
      @gui["menu.File/Exit"] = FXMenuCommand.new(pane, "E&xit\t\tClose all files and exit the application.")
			@gui["menu.File/Exit"].connect(SEL_COMMAND, method(:onQuit))
      
      @gui["menu.File"] = FXMenuTitle.new(@menubar, "&File", nil, pane)
      
      # edit menu
      
      pane = FXMenuPane.new(self)

      @gui["menu.Edit/Undo"] = FXMenuCommand.new(pane, "Undo\tCtl-Z\tUndo the last operation.", IconManager.load_icon(getApp, "undo.png"))
      @gui["menu.Edit/Undo"].connect(SEL_COMMAND, method(:onUndo))
      @gui["menu.Edit/Redo"] = FXMenuCommand.new(pane, "Redo\tCtl-Y\tRedo the last undone operation.", IconManager.load_icon(getApp, "redo.png"))
      @gui["menu.Edit/Redo"].connect(SEL_COMMAND, method(:onRedo))
      
      FXMenuSeparator.new(pane)
      
      @gui["menu.Edit/Cut"]     = FXMenuCommand.new(pane, "Cut\tCtl-X\tCut the selection to the clipboard.", IconManager.load_icon(getApp, "cut.png"))
      @gui["menu.Edit/Cut"].connect(SEL_COMMAND, method(:onCut))
      @gui["menu.Edit/Copy"]    = FXMenuCommand.new(pane, "Copy\tCtl-C\tCopy the selection to the clipboard.", IconManager.load_icon(getApp, "copy.png"))
      @gui["menu.Edit/Copy"].connect(SEL_COMMAND, method(:onCopy))
      @gui["menu.Edit/Paste"]   = FXMenuCommand.new(pane, "Paste\tCtl-V\tPaste the contents of the clipboard.", IconManager.load_icon(getApp, "paste.png"))
      @gui["menu.Edit/Paste"].connect(SEL_COMMAND, method(:onPaste))
      @gui["menu.Edit/Delete"]  = FXMenuCommand.new(pane, "Delete\tDel\tDelete the current selection.")
      @gui["menu.Edit/Delete"].connect(SEL_COMMAND, method(:onDelete))
      
      FXMenuSeparator.new(pane)
      
      @gui["menu.Edit/Select All"] = FXMenuCommand.new(pane, "Select All\tCtl-A\tSelect all objects in the current editor tab.")
      @gui["menu.Edit/Select All"].connect(SEL_COMMAND, method(:onSelectAll))
     
      FXMenuSeparator.new(pane)
      
      @gui["menu.Edit/Find..."]       = FXMenuCommand.new(pane, "Find...\tCtl-F\tSearch the current editor tab.", IconManager.load_icon(getApp, "find.png"))
			@gui["menu.Edit/Find..."].connect(SEL_COMMAND, method(:onFind))
      @gui["menu.Edit/Find Next"]     = FXMenuCommand.new(pane, "Find Next\tF3\tSearch again.")
			@gui["menu.Edit/Find Next"].connect(SEL_COMMAND, method(:onFindNext))
      @gui["menu.Edit/Find Previous"] = FXMenuCommand.new(pane, "Find Previous\tShift-F3\tSearch upwards.")
			@gui["menu.Edit/Find Previous"].connect(SEL_COMMAND, method(:onFindPrevious))
      @gui["menu.Edit/Replace..."]    = FXMenuCommand.new(pane, "Replace...\tCtl-H\tSearch and replace.")
      @gui["menu.Edit/Replace..."].connect(SEL_COMMAND, method(:onReplace))
      @gui["menu.Edit/Goto Line..."]  = FXMenuCommand.new(pane, "Goto Line...\tCtl-G\tGoto a specific line number.")
			@gui["menu.Edit/Goto Line..."].connect(SEL_COMMAND, method(:onGotoLine))
      
      FXMenuSeparator.new(pane)
			
      @gui["menu.Edit/Block Comment or Uncomment"] = FXMenuCommand.new(pane, "Block Comment or Uncomment\tCtl-Q\tComment or uncomment a range of text.")
      @gui["menu.Edit/Block Comment or Uncomment"].connect(SEL_COMMAND, method(:onBlockComment))
			
      FXMenuSeparator.new(pane)
			
      @gui["menu.Edit/Preferences..."] = FXMenuCommand.new(pane, "Preferences...\t\tEdit program preferences.")
      @gui["menu.Edit/Preferences..."].connect(SEL_COMMAND, method(:onEditPreferences))
      
			@gui["menu.Edit"] = FXMenuTitle.new(@menubar, "&Edit", nil, pane)
      
      # project menu
      
      pane = FXMenuPane.new(self)
      
      @gui["menu.Project/New File"] = FXMenuCommand.new(pane, "New File\t\tAdd a new file to the project.")
      @gui["menu.Project/New File"].connect(SEL_COMMAND, method(:onNewProjectFile))
      @gui["menu.Project/Add File(s)..."] = FXMenuCommand.new(pane, "Add File(s)...\t\tAdd files to the project.")
      @gui["menu.Project/Add File(s)..."].connect(SEL_COMMAND, method(:onAddFiles))
      @gui["menu.Project/Add Current File"] = FXMenuCommand.new(pane, "Add Current File\t\tAdd the current file to the active project.")
      @gui["menu.Project/Add Current File"].connect(SEL_COMMAND, method(:onAddCurrentFile))
      @gui["menu.Project/Remove Current File"] = FXMenuCommand.new(pane, "Remove Current File\t\tRemove the current file from its project.")
      @gui["menu.Project/Remove Current File"].connect(SEL_COMMAND, method(:onRemoveCurrentFile))

      FXMenuSeparator.new(pane)

      @gui["menu.Project/Set Current File As Project Main"] = FXMenuCommand.new(pane, "Set Current File As Project Main\t\tMake the current file the main project file.")
      @gui["menu.Project/Set Current File As Project Main"].connect(SEL_COMMAND, method(:onSetCurrentFileAsProjectMain))
			@gui["menu.Project/Project Settings..."] = FXMenuCommand.new(pane, "Project Settings...\t\tView the project settings dialog.")
      @gui["menu.Project/Project Settings..."].connect(SEL_COMMAND, method(:onProjectSettings))
      
      FXMenuSeparator.new(pane)
      
      @gui["menu.Project/Set Active Project"] = FXMenuCascade.new(pane, "Set Active Project\t\tChange the active project.", nil, FXMenuPane.new(self))
      
      @gui["menu.Project"] = FXMenuTitle.new(@menubar, "&Project", nil, pane)
      
      # run menu
      
      pane = FXMenuPane.new(self)
      
      @gui["menu.Run/Run File"]    = FXMenuCommand.new(pane, "Run File\tF9\tRun the active file.", IconManager.load_icon(getApp, "run.png"))
      @gui["menu.Run/Run File"].connect(SEL_COMMAND, method(:onRunFile))
      @gui["menu.Run/Run Project"] = FXMenuCommand.new(pane, "Run Project\tF5\tRun the active project.", IconManager.load_icon(getApp, "run_project.png"))
      @gui["menu.Run/Run Project"].connect(SEL_COMMAND, method(:onRunProject))
#      @gui["menu.Run/Stop Execution"] = FXMenuCommand.new(pane, "Stop Execution\tCtrl-Break\tStop the currently running file.")
#      @gui["menu.Run/Stop Execution"].connect(SEL_COMMAND, method(:onStopExecution))
#			@gui["menu.Run/Stop Execution"].disable
      
      @gui["menu.Run"] = FXMenuTitle.new(@menubar, "&Run", nil, pane)
      
      # tools menu
      
      pane = FXMenuPane.new(self)
      
      @gui["menu.Tools/Edit Environment..."] = FXMenuCommand.new(pane, "Edit Environment...\t\tEdit the file's execution environment.")
			@gui["menu.Tools/Edit Environment..."].connect(SEL_COMMAND, method(:onEditEnvironment))
      @gui["menu.Tools/Edit Parameters..."] = FXMenuCommand.new(pane, "Edit Parameters...\t\tEdit the file's command-line arguments.")
			@gui["menu.Tools/Edit Parameters..."].connect(SEL_COMMAND, method(:onEditParameters))
      @gui["menu.Tools/Rename File..."] = FXMenuCommand.new(pane, "Rename File...\t\tRename the file.")
			@gui["menu.Tools/Rename File..."].connect(SEL_COMMAND, method(:onRenameFile))

      @gui["menu.Tools"] = FXMenuTitle.new(@menubar, "&Tools", nil, pane)
			
      # view menu
      
      pane = FXMenuPane.new(self)
      
      @gui["menu.View/Left Dock"]		= FXMenuCheck.new(pane, "Left Dock\tF6\tView or hide the right dock frame.")
      @gui["menu.View/Left Dock"].connect(SEL_COMMAND, method(:onViewSidebar))
      @gui["menu.View/Right Dock"]	= FXMenuCheck.new(pane, "Right Dock\tF7\tView or hide the left dock frame.")
      @gui["menu.View/Right Dock"].connect(SEL_COMMAND, method(:onViewSidebar))
      @gui["menu.View/Bottom Dock"]	= FXMenuCheck.new(pane, "Bottom Dock\tF8\tView or hide the bottom dock frame.")
      @gui["menu.View/Bottom Dock"].connect(SEL_COMMAND, method(:onViewSidebar))
      @gui["menu.View/Home Page"]		= FXMenuCheck.new(pane, "Home Page\t\tView or hide the application home page.")
      @gui["menu.View/Home Page"].connect(SEL_COMMAND, method(:onViewHome))
      
      @gui["menu.View"] = FXMenuTitle.new(@menubar, "&View", nil, pane)
      
      # help menu
      
      pane = FXMenuPane.new(self)
      
      @gui["menu.Help/About Mondrian..."] = FXMenuCommand.new(pane, "About Mondrian...\t\tAbout the application.")
			@gui["menu.Help/About Mondrian..."].connect(SEL_COMMAND, method(:onAbout))
			
      @gui["menu.Help"] = FXMenuTitle.new(@menubar, "&Help", nil, pane)      
    end
    
    
    def sync_capabilities(capability = -1)
			if capability == -1
				#
				# enable/disable global commands
				#

				if SessionManager.instance.project_count > 0
					@gui["menu.File/Close Project"].enable
					@gui["menu.Project/New File"].enable
					@gui["menu.Project/Add File(s)..."].enable
					@gui["menu.Project/Set Active Project"].enable
					@gui["menu.Project/Project Settings..."].enable
				else
					@gui["menu.File/Close Project"].disable
					@gui["menu.Project/New File"].disable
					@gui["menu.Project/Add File(s)..."].disable
					@gui["menu.Project/Set Active Project"].disable
					@gui["menu.Project/Project Settings..."].disable
				end
				
				if FileManager.instance.open_files > 0
					@gui["toolbar.Search Files..."].enable
				else
					@gui["toolbar.Search Files..."].disable
				end
					
				if @focused_viewer and @focused_viewer.capable?(CMD_FILE)
					@gui["menu.File/Save File"].enable
					@gui["toolbar.Save File"].enable
					@gui["menu.File/Save File As..."].enable
					@gui["menu.Tools/Rename File..."].enable
					@gui["menu.Tools/Edit Parameters..."].enable
					@gui["menu.Tools/Edit Environment..."].enable

					if SessionManager.instance.get_associated_project(@focused_viewer.file)
						@gui["menu.File/Remove File From Workspace"].disable
						@gui["menu.Project/Add Current File"].disable
						@gui["menu.Project/Remove Current File"].enable
						if SessionManager.instance.get_associated_project(@focused_viewer.file).is_project_key?(@focused_viewer.file)
							@gui["menu.Project/Set Current File As Project Main"].disable
						else
							@gui["menu.Project/Set Current File As Project Main"].enable
						end
					else
						@gui["menu.File/Remove File From Workspace"].enable
						if SessionManager.instance.project_count > 0
							@gui["menu.Project/Add Current File"].enable
						else
							@gui["menu.Project/Add Current File"].disable
						end
						@gui["menu.Project/Remove Current File"].disable
						@gui["menu.Project/Set Current File As Project Main"].disable
					end
				else
					@gui["menu.File/Remove File From Workspace"].disable
					
					@gui["menu.File/Save File"].disable
					@gui["toolbar.Save File"].disable
					@gui["menu.File/Save File As..."].disable
					@gui["menu.Tools/Rename File..."].disable
					@gui["menu.Tools/Edit Parameters..."].disable
					@gui["menu.Tools/Edit Environment..."].disable

					@gui["menu.Project/Add Current File"].disable
					@gui["menu.Project/Remove Current File"].disable
					@gui["menu.Project/Set Current File As Project Main"].disable
				end
					
				if ExecutionManager.instance.is_running?
					@gui["menu.Run/Run File"].disable
					@gui["toolbar.Run File"].disable
					@gui["menu.Run/Run Project"].disable
					@gui["toolbar.Run Project"].disable
	#        @gui["menu.Run/Stop Execution"].enable
				else
					if @focused_viewer and @focused_viewer.capable?(CMD_FILE)
						@gui["menu.Run/Run File"].enable           
						@gui["toolbar.Run File"].enable           
					else
						@gui["menu.Run/Run File"].disable
						@gui["toolbar.Run File"].disable
					end
					if SessionManager.instance.project_count > 0
						@gui["menu.Run/Run Project"].enable
						@gui["toolbar.Run Project"].enable
					else
						@gui["menu.Run/Run Project"].disable
						@gui["toolbar.Run Project"].disable
					end
	#        @gui["menu.Run/Stop Execution"].disable
				end
				
				if @focused_viewer
					@gui["menu.File/Close File"].enable
					@gui["toolbar.Close File"].enable
				else
					@gui["menu.File/Close File"].disable
					@gui["toolbar.Close File"].disable
				end
			end
      
      #
      # enable/disable based on the currently active viewer
      #
			
			if @focused_viewer and @focused_viewer.capable?(GUI_CURSOR_POSITION)
				position = @focused_viewer.get_cursor_position
				
				@gui["statusbar.Line"].text 	= position[0].to_s
				@gui["statusbar.Column"].text = position[1].to_s
			else
				@gui["statusbar.Line"].text 	= "N/A"
				@gui["statusbar.Column"].text = "N/A"
			end
      
			if capability == -1 or capability == CMD_SELECT_ALL
				if @focused_viewer and @focused_viewer.capable?(CMD_SELECT_ALL)
					@gui["menu.Edit/Select All"].enable
				else
					@gui["menu.Edit/Select All"].disable
				end
			end
      
			if capability == -1 or capability == CMD_BLOCK_COMMENT
				if @focused_viewer and @focused_viewer.capable?(CMD_BLOCK_COMMENT)
					@gui["menu.Edit/Block Comment or Uncomment"].enable
				else
					@gui["menu.Edit/Block Comment or Uncomment"].disable
				end
			end
			
			if capability == -1 or capability == CMD_CUT
				if @focused_viewer and @focused_viewer.capable?(CMD_CUT)
					@gui["menu.Edit/Cut"].enable
					@gui["toolbar.Cut"].enable
				else
					@gui["menu.Edit/Cut"].disable
					@gui["toolbar.Cut"].disable
				end
			end
      
			if capability == -1 or capability == CMD_COPY
				if @focused_viewer and @focused_viewer.capable?(CMD_COPY)
					@gui["menu.Edit/Copy"].enable
					@gui["toolbar.Copy"].enable
				else
					@gui["menu.Edit/Copy"].disable
					@gui["toolbar.Copy"].disable
				end
			end
      
			if capability == -1 or capability == CMD_FIND
				if @focused_viewer and @focused_viewer.capable?(CMD_FIND)
					@gui["menu.Edit/Find..."].enable
					@gui["menu.Edit/Find Next"].enable
					@gui["menu.Edit/Find Previous"].enable
					@gui["menu.Edit/Replace..."].enable
					@gui["toolbar.Find..."].enable
				else
					@gui["menu.Edit/Find..."].disable
					@gui["menu.Edit/Find Next"].disable
					@gui["menu.Edit/Find Previous"].disable
					@gui["menu.Edit/Replace..."].disable
					@gui["toolbar.Find..."].disable
				end
			end
      
			if capability == -1 or capability == CMD_GOTO_LINE
				if @focused_viewer and @focused_viewer.capable?(CMD_GOTO_LINE)
					@gui["menu.Edit/Goto Line..."].enable
				else
					@gui["menu.Edit/Goto Line..."].disable
				end
			end
      
			if capability == -1 or capability == CMD_PASTE
				if @focused_viewer and @focused_viewer.capable?(CMD_PASTE)
					@gui["menu.Edit/Paste"].enable
					@gui["toolbar.Paste"].enable
				else
					@gui["menu.Edit/Paste"].disable
					@gui["toolbar.Paste"].disable
				end
			end
      
			if capability == -1 or capability == CMD_DELETE
				if @focused_viewer and @focused_viewer.capable?(CMD_DELETE)
					@gui["menu.Edit/Delete"].enable
				else
					@gui["menu.Edit/Delete"].disable
				end
			end
      
			if capability == -1 or capability == CMD_BACK
				if @focused_viewer and @focused_viewer.capable?(CMD_BACK)
					@gui["toolbar.Back"].enable
				else
					@gui["toolbar.Back"].disable
				end
			end

			if capability == -1 or capability == CMD_FORWARD
				if @focused_viewer and @focused_viewer.capable?(CMD_FORWARD)
					@gui["toolbar.Forward"].enable
				else
					@gui["toolbar.Forward"].disable
				end
			end

			if capability == -1 or capability == CMD_UNDO
				if @focused_viewer and @focused_viewer.capable?(CMD_UNDO)
					@gui["menu.Edit/Undo"].enable
					@gui["toolbar.Undo"].enable
				else
					@gui["menu.Edit/Undo"].disable
					@gui["toolbar.Undo"].disable
				end
			end
      
			if capability == -1 or capability == CMD_REDO
				if @focused_viewer and @focused_viewer.capable?(CMD_REDO)
					@gui["menu.Edit/Redo"].enable
					@gui["toolbar.Redo"].enable
				else
					@gui["menu.Edit/Redo"].disable
					@gui["toolbar.Redo"].disable
				end
			end
    end
    
		
    def create
      super

      # load previous session variables and set the GUI the way it was last session      
      
      session = SessionManager.instance
      
      main_x      = session.restore("main_x")
      main_y      = session.restore("main_y")      
      main_width  = session.restore("main_width")
      main_height = session.restore("main_height")
      
      dock_bottom_height  = session.restore("dock_bottom_height")
      dock_bottom_hidden = session.restore("dock_bottom_hidden")
      @dock_bottom.height = dock_bottom_height if dock_bottom_height != nil
      @dock_bottom.handle(self, MKUINT(ID_HIDE, SEL_COMMAND), nil) if dock_bottom_hidden
      
      dock_left_width  = session.restore("dock_left_width")
      dock_left_hidden = session.restore("dock_left_hidden")
      @dock_left.width = dock_left_width if dock_left_width != nil
      @dock_left.handle(self, MKUINT(ID_HIDE, SEL_COMMAND), nil) if dock_left_hidden

      dock_right_width  = session.restore("dock_right_width")
      dock_right_hidden = session.restore("dock_right_hidden")
      @dock_right.width = dock_right_width if dock_right_width != nil
      @dock_right.handle(self, MKUINT(ID_HIDE, SEL_COMMAND), nil) if dock_right_hidden
      
      if main_x and main_y and main_width and main_height
        position(main_x, main_y, main_width, main_height)
        show
      else
        show(PLACEMENT_SCREEN)
      end
      
			sourceBrowser_mode	= session.restore("sourceBrowser_mode")
			@sourceBrowser.mode = sourceBrowser_mode if sourceBrowser_mode != nil
			
      #
      # post-GUI initialization
      #

      @sourceBrowser.slave  = @editBook
      @reportFrame.slave    = @editBook
			@searchFrame.slave		= @editBook
      
      @editBook.add_observer(self)

			register_default_viewers      
      
      launch_home_page
			
			apply_preferences
			
			#
			# handle command-line parameters
			#
			
			if ARGV[0]
				begin
					file = FileManager.instance.open_file(ARGV[0])
					if file.kind_of?(ProjectFile)
						SessionManager.instance.open_project(file)
					else
						@editBook.launch_viewer(nil, file)
					end
				rescue
					FXMessageBox.error(self, MBOX_OK, "Parameter Error", $!)
				end
			end
    end
    
    
		def apply_preferences
			set_default_preferences # this fixes any incorrect values but leaves changed values alone
		
			session = SessionManager.instance
			
			ExecutionManager.instance.capture_stdout 	= session.restore("Preferences/Capture Stdout")
			ExecutionManager.instance.terminal_string = session.restore("Preferences/Terminal String")
			
			@editBook.each_viewer { |viewer|
				viewer.refresh
			}
			
			getApp.removeTimeout(@save_timer) if defined?(@save_timer)
			@save_timer = getApp.addTimeout(session.restore("Preferences/Timed Save") * 60 * 1000, method(:onTimedSave)) if session.restore("Preferences/Timed Save") > 0
		end
		
		
		def set_default_preferences(override = false)
			session = SessionManager.instance
			
			session.store("Preferences/Timed Save", 0) if session.restore("Preferences/Timed Save") == nil or override
			session.store("Preferences/Save On Tab Switch", false) if session.restore("Preferences/Save On Tab Switch") == nil or override
			
			session.store("Preferences/Ruby Path", "ruby") if session.restore("Preferences/Ruby Path") == nil or session.restore("Preferences/Ruby Path") == "" or override
			
			session.store("Preferences/Auto Save", false) if session.restore("Preferences/Auto Save") == nil or override
			session.store("Preferences/Capture Stdout", false) if session.restore("Preferences/Capture Stdout") == nil or override
			
			session.store("Preferences/Line Numbers", false) if session.restore("Preferences/Line Numbers") == nil or override
			session.store("Preferences/EOL Markers", false) if session.restore("Preferences/EOL Markers") == nil or override
			session.store("Preferences/Convert Tabs", false) if session.restore("Preferences/Convert Tabs") == nil or override
			session.store("Preferences/Tab Width", 2) if session.restore("Preferences/Tab Width") == nil or override
			session.store("Preferences/Show Change Tracker", false) if session.restore("Preferences/Show Change Tracker") == nil or override
			
			if RUBY_PLATFORM =~ /win32/
				session.store("Preferences/Web Browser", "Internet Explorer") if session.restore("Preferences/Web Browser") == nil or session.restore("Preferences/Web Browser") == "" or override
				session.store("Preferences/Terminal String", "") or override
			else
				session.store("Preferences/Web Browser", "netscape") if session.restore("Preferences/Web Browser") == nil or session.restore("Preferences/Web Browser") == "" or override
				session.store("Preferences/Terminal String", "xterm -e") if session.restore("Preferences/Terminal String") == nil or session.restore("Preferences/Terminal String") == "" or override
			end
			
			session.store("Preferences/Use Custom Styles", false) if session.restore("Preferences/Use Custom Styles") == nil or override
			session.store("Preferences/Editor Styles", EditorStyles.new) if session.restore("Preferences/Editor Styles") == nil or override
		end
		
		
		def onEditPreferences(sender, message, data)
      dlg = PreferencesDialog.new(self)
			
			dlg.execute(PLACEMENT_SCREEN)
		end
		
		
    def onCloseFile(sender, message, data)
      @editBook.close_current_viewer
    end


    def onRemoveFileFromWorkspace(sender, message, data)
      @focused_viewer.file.close
    end
		
		
    def onQuit(sender, message, data)
			# close all open files and projects
			FileManager.instance.each(ProjectFile) { |project| return unless project.close }
			FileManager.instance.each { |file| return unless file.close }
		
			# save session data
      session = SessionManager.instance
      
			unless isMinimized
				session.store("main_x", x)
				session.store("main_y", y)
				session.store("main_width", width)
				session.store("main_height", height)
			end
      
      session.store("dock_bottom_height", @dock_bottom.height)
      session.store("dock_bottom_hidden",  !@dock_bottom.shown?)
      session.store("dock_left_width", @dock_left.width)
      session.store("dock_left_hidden", !@dock_left.shown?)
      session.store("dock_right_width", @dock_right.width)
      session.store("dock_right_hidden", !@dock_right.shown?)
			
			session.store("sourceBrowser_mode", @sourceBrowser.mode)
      
      session.save
      
      # quit the application
      getApp.handle(self, MKUINT(FXApp::ID_QUIT, SEL_COMMAND), nil)    end
    
    
	def onOpenFile(sender, message, data)
		openDlg = FXFileDialog.new(self, "Open File")
		
		openDlg.selectMode      = SELECTFILE_EXISTING | SELECTFILE_MULTIPLE
		openDlg.patternList     = ["All Files (*)", "Ruby Files (*.rb,*.rbw)"]
		openDlg.currentPattern  = 1
      
		if openDlg.execute(PLACEMENT_SCREEN) != 0
	
			openDlg.filenames.each do |filename|
				begin
					file = FileManager.instance.open_file(filename)
				rescue
					FXMessageBox.error(FXApp.instance, MBOX_OK, "Open File", $!.message)
					return
				end
	
				if file
					SessionManager.instance.set_working_directory(file.directory)
					
					if file.kind_of?(RubyFile)
						@editBook.launch_viewer(nil, file)
					else
						FXMessageBox.error(SessionManager.instance.main_window, MBOX_OK, "Open File", "Unsupported file type for this operation.")
						file.close
					end
				end
			end		
		end			
	end
    
    
    def onNewFile(sender, message, data)
      @editBook.launch_viewer(RubyText, nil)
    end
    
    
    def onNewProject(sender, message, data)
      newDlg = CreateProjectDialog.new(self)
      
      if newDlg.execute(PLACEMENT_SCREEN) == 1
        SessionManager.instance.open_project(newDlg.project)
      end
    end
    
    
    def onOpenProject(sender, message, data)
      # prompt the user to select a project file
      
      openDlg = FXFileDialog.new(self, "Open Project")
      
      openDlg.selectMode      = SELECTFILE_EXISTING
      openDlg.patternList     = ["All Files (*)", "Project Files (*.mpr)"]
      openDlg.currentPattern  = 1
 
      if openDlg.execute(PLACEMENT_SCREEN) != 0      
				begin
					project = FileManager.instance.open_file(openDlg.filename)
				rescue
					FXMessageBox.error(SessionManager.instance.main_window, MBOX_OK, "Open Project", $!)
					return
				end

        unless project.kind_of?(ProjectFile)
          FXMessageBox.error(self, MBOX_OK, "Open Project", "'#{openDlg.filename}' is not a project file.")
					return
        end
      end
      
      SessionManager.instance.open_project(project) if project
    end
    
    
		def onCloseProject(sender, message, data)
			SessionManager.instance.active_project.close
		end
		
		
    def onAddFiles(sender, message, data)
			SessionManager.instance.active_project.add_files if SessionManager.instance.active_project
    end
    
		
    def onNewProjectFile(sender, message, data)
			SessionManager.instance.active_project.new_file if SessionManager.instance.active_project
    end
		
		
		def onProjectSettings(sender, message, data)
			SessionManager.instance.active_project.project_settings if SessionManager.instance.active_project
		end
		
    
		def onAddCurrentFile(sender, message, data)
			begin
				SessionManager.instance.active_project.add_file(@focused_viewer.file) if 
					SessionManager.instance.active_project and @focused_viewer and @focused_viewer.file
			rescue
				FXMessageBox.error(SessionManager.instance.main_window, MBOX_OK, "Add Current File", "Could not add file '#{@focused_viewer.file.name}'.\nThe file needs to be saved first.")
			end

			sync_capabilities

			@focused_viewer.setFocus if @focused_viewer
		end
		
		
		def onRemoveCurrentFile(sender, message, data)
      SessionManager.instance.get_associated_project(@focused_viewer.file).remove_file(@focused_viewer.file) if 
				SessionManager.instance.get_associated_project(@focused_viewer.file) and @focused_viewer and @focused_viewer.file

			sync_capabilities

			@focused_viewer.setFocus if @focused_viewer
		end
		
		
		def onSetCurrentFileAsProjectMain(sender, message, data)
      SessionManager.instance.get_associated_project(@focused_viewer.file).project_key = @focused_viewer.file if
				SessionManager.instance.get_associated_project(@focused_viewer.file) and @focused_viewer and @focused_viewer.file
			
			sync_capabilities
			
			@focused_viewer.setFocus if @focused_viewer
		end
		
		
    def onRunFile(sender, message, data)
      @focused_viewer.file.run if @focused_viewer and @focused_viewer.file
			
			@focused_viewer.setFocus if @focused_viewer
    end
		
		
		def onRunProject(sender, message, data)
			SessionManager.instance.active_project.run if SessionManager.instance.active_project
		end
    
    
    def onStopExecution(sender, message, data)
      ExecutionManager.instance.stop_execution
    end
    
    
    def onForward(sender, message, data)
			@focused_viewer.invoke(CMD_FORWARD, data)
    end


    def onBack(sender, message, data)
			@focused_viewer.invoke(CMD_BACK, data)
    end


    def onUndo(sender, message, data)
			@focused_viewer.invoke(CMD_UNDO, data)
    end
    
    
    def onRedo(sender, message, data)
			@focused_viewer.invoke(CMD_REDO, data)
    end
    
    
    def onCut(sender, message, data)
			@focused_viewer.invoke(CMD_CUT, data)
    end
    
    
    def onCopy(sender, message, data)
			@focused_viewer.invoke(CMD_COPY, data)
    end
    
    
    def onPaste(sender, message, data)
			@focused_viewer.invoke(CMD_PASTE, data)
    end
    
    
    def onFind(sender, message, data)
			@focused_viewer.invoke(CMD_FIND)
    end
		
		
    def onSearchFiles(sender, message, data)
			SearchFilesDialog.instance.execute
    end
		
		
    def onReplace(sender, message, data)
      @focused_viewer.invoke(CMD_REPLACE) if @focused_viewer
    end
		
		
		def onQuickFind(sender, message, data)
			return if data == "" or 
								!@gui["toolbar.Quick Find"].children[0].hasFocus? # hack...combobox has no TEXTFIELD_ENTER_ONLY option
			
			@focused_viewer.invoke(CMD_FIND_NEXT, data) if @focused_viewer and @focused_viewer.capable?(GUI_QUICK_FIND)
			
			(@gui["toolbar.Quick Find"].numItems - 1).step(0, -1) { |index| 
				@gui["toolbar.Quick Find"].removeItem(index) if @gui["toolbar.Quick Find"].getItem(index) == data
			}
			
			@gui["toolbar.Quick Find"].prependItem(data)
			@gui["toolbar.Quick Find"].currentItem = 0
			
			@gui["toolbar.Quick Find"].first.setFocus
			@gui["toolbar.Quick Find"].children[0].handle(nil, MKUINT(FXTextField::ID_CURSOR_END, SEL_COMMAND), nil) # the cursor gets screwed up when removing/adding items
		end
		
		
    def onFindNext(sender, message, data)
			@focused_viewer.invoke(CMD_FIND_NEXT) if @focused_viewer
    end
    
		
		def onFindPrevious(sender, message, data)
			@focused_viewer.invoke(CMD_FIND_PREVIOUS) if @focused_viewer
    end
		
		
		def onGotoLine(sender, message, data)
			@focused_viewer.invoke(CMD_GOTO_LINE) if @focused_viewer
		end
		
		
    def onDelete(sender, message, data)
			@focused_viewer.invoke(CMD_DELETE) if @focused_viewer
    end
  
    
    def onSelectAll(sender, message, data)
			@focused_viewer.invoke(CMD_SELECT_ALL) if @focused_viewer
    end
    
    
    def onBlockComment(sender, message, data)
			@focused_viewer.invoke(CMD_BLOCK_COMMENT) if @focused_viewer
    end
		
		
    def onSave(sender, message, data)
			@focused_viewer.file.save if @focused_viewer and @focused_viewer.file
			
			@focused_viewer.setFocus if @focused_viewer
    end
    
    
    def onSaveAs(sender, message, data)
			@focused_viewer.file.save_as if @focused_viewer and @focused_viewer.file
			
			@focused_viewer.setFocus if @focused_viewer
    end
		
		
		def onSaveAll(sender, message, data)
			FileManager.instance.each { |file| file.save }
			
			@focused_viewer.setFocus if @focused_viewer
		end
		
		
		def onTimedSave(sender, message, data)
			FileManager.instance.each { |file| file.save unless file.is_new? or file.clean? }
			
			@save_timer = getApp.addTimeout(SessionManager.instance.restore("Preferences/Timed Save") * 60 * 1000, method(:onTimedSave)) if SessionManager.instance.restore("Preferences/Timed Save") > 0
		end
		
		
		def onEditParameters(sender, message, data)
			@focused_viewer.file.edit_parameters if @focused_viewer and @focused_viewer.file
			
			@focused_viewer.setFocus if @focused_viewer
		end
    
    
		def onEditEnvironment(sender, message, data)
			@focused_viewer.file.edit_environment if @focused_viewer and @focused_viewer.file
			
			@focused_viewer.setFocus if @focused_viewer
		end
		
		
		def onRenameFile(sender, message, data)
			@focused_viewer.file.rename if @focused_viewer and @focused_viewer.file
			
			@focused_viewer.setFocus if @focused_viewer
		end
		
		
    def onViewHome(sender, message, data)
      if sender.checked?
        close_home_page
      else
        launch_home_page
      end
    end
		
		
		def onAbout(sender, message, data)
      dlg = FXDialogBox.new(self, "About Mondrian", DECOR_TITLE | DECOR_BORDER | DECOR_CLOSE, 0, 0, 0, 0)
			dlg.backColor = "White"
      
      frame = FXVerticalFrame.new(dlg, FRAME_NONE, 0, 0, 0, 0)
			frame.backColor = "White"
			
			group_box = FXGroupBox.new(frame, "Program Details", GROUPBOX_NORMAL | FRAME_LINE | LAYOUT_FILL_X)
			group_box.backColor = "White"
			
			f = FXHorizontalFrame.new(group_box, FRAME_NONE, 0, 0, 0, 0, 0, 0, 0, 0)
			f.backColor = "White"

			iconView    = FXImageView.new(f, nil, nil, 0, LAYOUT_FIX_HEIGHT | LAYOUT_FIX_WIDTH | LAYOUT_CENTER_Y)
			iconImage   = FXBMPImage.new(parent.getApp, nil, IMAGE_KEEP)
			FXFileStream.open(SessionManager.instance.program_directory + File::Separator + "images" + File::Separator + "mondrian_icon.bmp", FXStreamLoad) do |stream| 
				iconImage.loadPixels(stream)
			end
			iconView.width  = iconImage.width
			iconView.height = iconImage.height
			iconView.image  = iconImage

			f = FXVerticalFrame.new(f, FRAME_NONE, 0, 0, 0, 0, 0, 0, 0, 0)
			f.backColor = "White"

			FXLabel.new(f, SessionManager.instance.program_name_with_version, nil, LABEL_NORMAL).backColor = "White"
			FXLabel.new(f, "Email support at #{SessionManager.instance.support_email_address}.").backColor = "White"

			FXLabel.new(group_box, "Get help or information at #{SessionManager.instance.product_homepage}.").backColor = "White"
			FXLabel.new(group_box, "Copyright (c) 2005, #{SessionManager.instance.company_name}.").backColor = "White"
			FXLabel.new(group_box, "Released under the MIT License.").backColor = "White"
			
			group_box = FXGroupBox.new(frame, "Supporting Software (thanks!)", GROUPBOX_NORMAL | FRAME_LINE | LAYOUT_FILL_X)
			group_box.backColor = "White"
			
			FXLabel.new(group_box, "Ruby v#{RUBY_VERSION} is copyright (c) Yukihiro Matsumoto").backColor = "White"
			FXLabel.new(group_box, "The FOX GUI toolkit v#{Fox::fxversion} is copyright (c) Jeroen van der Zijp").backColor = "White"
			FXLabel.new(group_box, "FXRuby v#{Fox::fxrubyversion} is copyright (c) J. Lyle Johnson").backColor = "White"
			FXLabel.new(group_box, "Scintilla is copyright (c) Neil Hodgson").backColor = "White"
			FXLabel.new(group_box, "FXScintilla is copyright (c) Gilles Filippini").backColor = "White"
			
			frame = FXHorizontalFrame.new(dlg, FRAME_NONE | LAYOUT_CENTER_X, 0, 0, 0, 0)
			frame.backColor = "White"
			
      button = FXButton.new(frame, "OK", nil, dlg, FXDialogBox::ID_ACCEPT, BUTTON_NORMAL | LAYOUT_FIX_WIDTH)
      button.resize(70, 0)
			button.setFocus
			
			alive = true
			
			rotate = Proc.new { 
				if alive
					iconView.image.rotate([90, 180, 270][rand(3)])
					iconView.update
					FXApp.instance.addTimeout(2000) { rotate.call }
				end
			}
			
			rotate.call
			
			dlg.execute(PLACEMENT_OWNER)
			
			alive = false
		end
    
    
    def launch_home_page
      @editBook.launch_viewer(HomeViewer)
      @gui["menu.View/Home Page"].check = true
    end
    
    
    def close_home_page
      @editBook.each_viewer(HomeViewer) { |home| @editBook.close_viewer(home) }
      @gui["menu.View/Home Page"].check = false
    end
    
    
    def onViewSidebar(sender, message, data)
      case sender
        when @gui["menu.View/Left Dock"]
          sidebar = @dock_left
        when @gui["menu.View/Right Dock"]
          sidebar = @dock_right
        when @gui["menu.View/Bottom Dock"]
          sidebar = @dock_bottom
      end
      
      return unless sidebar
      
      if sender.checked?
        sidebar.handle(self, MKUINT(ID_HIDE, SEL_COMMAND), nil)
      else
        sidebar.handle(self, MKUINT(ID_SHOW, SEL_COMMAND), nil)
      end
    end
    
    
    def onSidebarChange(sender, message, data)
      case sender
        when @dock_left
          menu = @gui["menu.View/Left Dock"]
        when @dock_right
          menu = @gui["menu.View/Right Dock"]
        when @dock_bottom
          menu = @gui["menu.View/Bottom Dock"]
      end
      
      return unless menu

      if sender.shown?
        menu.check = true
      else
        menu.check = false
      end
    end
    
    
    def update(sender, message, data)
      case sender
        when @focused_viewer
          case message
            when CAPABILITY_ADDED, CAPABILITY_REMOVED
              sync_capabilities(data)
							
						when CAPABILITY_UPDATE
							case data[0]
								when GUI_CURSOR_POSITION
									@gui["statusbar.Line"].text 	= data[1][0].to_s
									@gui["statusbar.Column"].text = data[1][1].to_s
							end
          end
        
        when @editBook
          case message
            when VIEWER_CLOSED 
							# kind of a hack to uncheck the View Home Page option
              @gui["menu.View/Home Page"].check = false if data.instance_of?(HomeViewer)

							if SessionManager.instance.restore("Preferences/Save On Tab Switch") and
									data.capable?(CMD_FILE)
								file = data.file
								file.save if file and file.dirty? and not file.is_new?
							end
							
            when FOCUS_CHANGED
							if SessionManager.instance.restore("Preferences/Save On Tab Switch") and
									@focused_viewer and 
									@focused_viewer.capable?(CMD_FILE)
								file = @focused_viewer.file
								file.save if file and file.dirty? and not file.is_new?
							end
							
							@focused_viewer.delete_observer(self) if @focused_viewer
              @focused_viewer = data
              @focused_viewer.add_observer(self) if @focused_viewer
								
              sync_capabilities # enable/disable menu items and toolbar buttons
          end
        
        when ExecutionManager.instance
          case message
            when EXECUTION_BEGIN, EXECUTION_END
              sync_capabilities
          end
        
        when FileManager.instance
          case message
            when FILE_OPENED    # catch open file requests and parse the file...it would
              parse_file(data)  # happen anyway, but this lets us throw up a wait cursor
          end
          
        when SessionManager.instance
          case message
            when PROJECT_OPENED
              # create a new project item in the 'Set Active Project' menu
              command = FXMenuCheck.new(@gui["menu.Project/Set Active Project"].getMenu, "#{data.project_name}\t\tSet the currently active project.")
              @gui["menu.Project/Set Active Project/#{data.project_name}"] = command
              command.connect(SEL_COMMAND) { SessionManager.instance.set_active_project(data) }
              command.create
              
              sync_capabilities # activate the project menu/toolbar commands
              
              @editBook.launch_viewer(nil, data)
            
            when PROJECT_REOPENED
              @editBook.launch_viewer(nil, data)
							
            when ACTIVE_PROJECT_CHANGED
              @gui["menu.Project/Set Active Project/#{data.project_name}"].check = false if data
              @gui["menu.Project/Set Active Project/#{SessionManager.instance.active_project.project_name}"].check = true if SessionManager.instance.active_project
						
						when PROJECT_CLOSED
							@gui["menu.Project/Set Active Project"].getMenu.removeChild(@gui["menu.Project/Set Active Project/#{data.project_name}"])
              @gui.delete("menu.Project/Set Active Project/#{data.project_name}")

							sync_capabilities
							
						when STATUS_MESSAGE
							@statusbar.statusLine.text = data
							
						when PROJECT_FILE_ADDED, PROJECT_FILE_REMOVED
							sync_capabilities
          end
      end
    end
    
    
    def parse_file(file)
      return unless file.kind_of?(RubyFile) and not file.is_new?

      @statusbar.statusLine.text = "Parsing #{file.full_name}..."
      
      getApp.beginWaitCursor {
        parser = file.parser # trigger the parser to build itself
      }
      
      @statusbar.statusLine.text = ""
    end
  end

end



PROGRAM_DIRECTORY = File.expand_path(File.dirname(__FILE__))

$> = $stderr # set the default output to standard error

application = FXApp.new("Mondrian Ruby IDE", "Brillig Systems, Inc.")

# get rid of the default font boldness on linux
font 					= application.normalFont
fd 						= font.fontDesc
fd.weight			= FONTWEIGHT_REGULAR
fd.size 			= 87 unless RUBY_PLATFORM =~ /win32/
font.fontDesc	= fd

Mondrian::MainWindow.new(application)

application.create

begin
	application.run
rescue
	message = "Caught unhandled exception:\n\n'#{$!}\n\n#{$@.join("\n")}'\n\nPlease send bug reports to #{SessionManager.instance.support_email_address}.\n\nWould you like to try and save all files now?"
	errorBox = FXDialogBox.new(SessionManager.instance.main_window, "Fatal Error")
	FXLabel.new(errorBox, message, nil, JUSTIFY_LEFT)
	FXHorizontalFrame.new(errorBox, LAYOUT_CENTER_X) { |frame|
		FXButton.new(frame, "Yes", nil, errorBox, FXDialogBox::ID_ACCEPT, BUTTON_NORMAL | LAYOUT_FIX_WIDTH, 0, 0, 90)
		FXButton.new(frame, "No", nil, errorBox, FXDialogBox::ID_CANCEL, BUTTON_NORMAL | LAYOUT_FIX_WIDTH, 0, 0, 90)
	}	
	if errorBox.execute == FXDialogBox::ID_ACCEPT
		FileManager.instance.each { |file| file.save }
	end
	
	raise
end